using System;
using System.Collections;
using System.ComponentModel;

namespace C1Tdbg_Hierarchical_BusinessObject
{
	/// <summary>
	/// Customer class.
	/// </summary>
	class Customer
	{
		public string CustName = null;
        public IBindingList CustOrders = null;
	}

	/// <summary>
	/// Order class.
	/// </summary>
	class Order
	{
		public DateTime OrderDate;
        public IBindingList OrderItems;
	}

	/// <summary>
	/// OrderItem class.
	/// </summary>
	class OrderItem
	{
		public int ItemQuantity;
		public string ItemDescription;
	}

	/// <summary>
	/// CustomerList class: A collection of Customer objects that can be assigned
	/// as a grid's DataSource.
	/// </summary>
	class CustomerList : ArrayList, ITypedList
	{
		PropertyDescriptorCollection ITypedList.GetItemProperties(PropertyDescriptor[] listAccessors)
		{
			if (listAccessors == null)
			{
				// Return the property descriptors for top-level rows
				return new PropertyDescriptorCollection (new PropertyDescriptor[]
						{
							new MyPropertyDescriptor("CustName"),
							new MyPropertyDescriptor("CustOrders")
						});
			}
			else
			{
				// Return the property descriptors for second-level and third-level rows
				string parentDescriptorName = listAccessors[listAccessors.Length - 1].Name;
				switch (parentDescriptorName)
				{
					case "CustOrders":
						return new PropertyDescriptorCollection(new PropertyDescriptor[]
								{
									new MyPropertyDescriptor("OrderDate"),
									new MyPropertyDescriptor("OrderItems")
								});

					case "OrderItems":
						return new PropertyDescriptorCollection(new PropertyDescriptor[]
								{
									new MyPropertyDescriptor("ItemQuantity"),
									new MyPropertyDescriptor("ItemDescription")
								});

					default:
						throw new Exception("Not implemented: " + parentDescriptorName);
				}
			}
		}

		string ITypedList.GetListName(PropertyDescriptor[] listAccessors)
		{
			return "";
		}
	}

	/// <summary>
	/// Property descriptor implementation.
	/// Store only the name in the base class, then override key methods.
	/// Overridden methods look up the name in the base class to decide
	/// how to carry out each property descriptor function.
	/// </summary>
	class MyPropertyDescriptor : PropertyDescriptor
	{
		public MyPropertyDescriptor(string descriptorName)
			: base(descriptorName, new Attribute[0])
		{
		}

		public override Type ComponentType
		{
			get
			{
				switch (this.Name)
				{
					case "CustName":         return typeof(Customer);
					case "CustOrders":       return typeof(Customer);
					case "OrderDate":        return typeof(Order);
					case "OrderItems":       return typeof(Order);
					case "ItemQuantity":     return typeof(OrderItem);
					case "ItemDescription":  return typeof(OrderItem);
					default:                 return null;
				}
			}
		}

		public override Type PropertyType
		{
			get
			{
				switch (this.Name)
				{
					case "CustName":           return typeof(string);
					case "CustOrders":         return typeof(ArrayList);
					case "OrderDate":          return typeof(DateTime);
					case "OrderItems":         return typeof(ArrayList);
					case "ItemQuantity":       return typeof(int);
					case "ItemDescription":    return typeof(string);
					default:                   return null;
				}
			}
		}

		public override object GetValue(object component)
		{
			switch (this.Name)
			{
				case "CustName":         return ((Customer)component).CustName;
				case "CustOrders":       return ((Customer)component).CustOrders;
				case "OrderDate":        return ((Order)component).OrderDate;
				case "OrderItems":       return ((Order)component).OrderItems;
				case "ItemQuantity":     return ((OrderItem)component).ItemQuantity;
				case "ItemDescription":  return ((OrderItem)component).ItemDescription;
				default:                 return null;
			}
		}

		public override string DisplayName
		{
			get
			{
				switch (this.Name)
				{
					case "CustName":         return "Customer name";
					case "CustOrders":       return "Customer orders";
					case "OrderDate":        return "Order date";
					case "OrderItems":       return "Order items";
					case "ItemQuantity":     return "Quantity";
					case "ItemDescription":  return "Description";
					default:                 return null;
				}
			}
		}

		public override bool IsReadOnly { get { return true; } }
		public override void SetValue(object component, object value)  { throw new Exception("Not implemented."); }
		public override void ResetValue(object component)              { throw new Exception("Not implemented."); }
		public override bool CanResetValue(object component)           { throw new Exception("Not implemented."); }
		public override bool ShouldSerializeValue(object component)    { throw new Exception("Not implemented."); }
	}
}
